// -*- C++ -*-
/*!
 * @file  SpeechDetectorConsumer.cpp
 * @brief Speech Detector Consumer Component
 * $Date$ 
 *
 * $Id$ 
 */
#include "SpeechDetectorConsumer.h"

// Module specification
// <rtc-template block="module_spec">
static const char* speechdetectorconsumer_spec[] =
  {
    "implementation_id", "SpeechDetectorConsumer",
    "type_name",         "SpeechDetectorConsumer",
    "description",       "Speech Detector Consumer Component",
    "version",           "1.0",
    "vendor",            "Kyusyu Institute of Technology",
    "category",          "Consumer",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    ""
  };
// </rtc-template>

// u[Xgꂽފ̃JXP[h
static const char *cascade_name_face = "haarcascades\\haarcascade_frontalface_alt.xml"; // ʊ
static const char *cascade_name_eye = "haarcascades\\haarcascade_eyes.xml"; // 
static const char *cascade_name_nose = "haarcascades\\Nariz.xml"; // @
static const char *cascade_name_mouth = "haarcascades\\Boca.xml"; // 

SpeechDetectorConsumer::SpeechDetectorConsumer(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_SpeechDetectorPort("SpeechDetector")

    // </rtc-template>
{
	// Registration: InPort/OutPort/Service
	// <rtc-template block="registration">
	// Set InPort buffers

	// Set OutPort buffer

	// Set service provider to Ports

	// Set service consumers to Ports
	m_SpeechDetectorPort.registerConsumer("speechDetector", "SpeechDetector", m_speechDetector);

	// Set CORBA Service Ports
	registerPort(m_SpeechDetectorPort);

	// </rtc-template>

}

SpeechDetectorConsumer::~SpeechDetectorConsumer()
{
}


RTC::ReturnCode_t SpeechDetectorConsumer::onInitialize()
{
	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	// </rtc-template>

	m_capture = NULL;
	m_currentFrame = 0;
	m_beforeImage = NULL;
	m_resultImage = NULL;

	return RTC::RTC_OK;
}


RTC::ReturnCode_t SpeechDetectorConsumer::onFinalize()
{
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t SpeechDetectorConsumer::onStartup(RTC::UniqueId ec_id)
{
}
*/

/*
RTC::ReturnCode_t SpeechDetectorConsumer::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t SpeechDetectorConsumer::onActivated(RTC::UniqueId ec_id)
{
	if(NULL == (m_capture = cvCaptureFromCAM(0))){ // Jo
        printf("Not found the camera. Abort..."); // G[\
		return RTC::RTC_ERROR;
    }

    cvSetCaptureProperty(m_capture,CV_CAP_PROP_FRAME_WIDTH,320);
    cvSetCaptureProperty(m_capture,CV_CAP_PROP_FRAME_HEIGHT,240);

	// u[Xgꂽފ̃JXP[hǂݍ
	m_cascadeFace = (CvHaarClassifierCascade *)cvLoad(cascade_name_face, 0, 0, 0);
	m_cascadeEye = (CvHaarClassifierCascade *)cvLoad(cascade_name_eye, 0, 0, 0);
	m_cascadeNose = (CvHaarClassifierCascade *)cvLoad(cascade_name_nose, 0, 0, 0);
	m_cascadeMouth = (CvHaarClassifierCascade *)cvLoad(cascade_name_mouth, 0, 0, 0);

	// EBhE쐬
	cvNamedWindow("CaptureImage", CV_WINDOW_AUTOSIZE);

	return RTC::RTC_OK;
}

RTC::ReturnCode_t SpeechDetectorConsumer::onDeactivated(RTC::UniqueId ec_id)
{
	cvDestroyWindow("CaptureImage");
	if (m_resultImage != NULL) {
		cvReleaseImage(&m_resultImage);
	}
	if (m_beforeImage != NULL) {
		cvReleaseImage(&m_beforeImage);
	}
	if (m_capture != NULL) {
		cvReleaseCapture(&m_capture);
	}
	return RTC::RTC_OK;
}

RTC::ReturnCode_t SpeechDetectorConsumer::onExecute(RTC::UniqueId ec_id)
{
	int64 start_tick = 0;
	IplImage *captureImage = NULL;
	bool prev_speeching = m_speeching;
	
	start_tick = cvGetTickCount(); // ԌvJn
	m_currentFrame++;

	captureImage = cvQueryFrame(m_capture); // 摜Lv`
	cvFlip(captureImage, captureImage, 0); // 摜̏㉺]

	// wb_̍쐬ƃf[^̈̊m
	if (m_resultImage == NULL) {
		m_resultImage = cvCreateImage(cvSize(captureImage->width, captureImage->height),
			captureImage->depth, captureImage->nChannels);
	}
	// 摜̃Rs[
	cvCopy(captureImage, m_resultImage, NULL);

	//********** ʊyъ畔io **********
	m_fd.FaceDetect(captureImage, m_cascadeFace, m_cascadeEye, m_cascadeNose, m_cascadeMouth);

	//********** oʕ` **********
	m_fd.RectDraw(m_resultImage);
	cvShowImage("CaptureImage", m_resultImage); // oʕ\
	cvWaitKey(1); // ꂪƕ`҂݂

	// ołꍇ̂ݔbs
	if(m_fd.MouthResult()){
		if (m_beforeImage != NULL) {
			CvRect mouth, mouth_before; // ̈it[COt[j

			// ̈擾
			mouth = m_fd.GetMouth();
			mouth_before = m_fdBefore.GetMouth();

			//********** ʎZo **********
			m_ueData.OF(captureImage, m_beforeImage, mouth, mouth_before);
			m_ueData.SAD(captureImage, m_beforeImage, mouth, mouth_before);

			//fprintf(wfp,"OF:%lf\t", ue_data.GetOF()); // ʂt@Cɏ
			//fprintf(wfp,"SAD:%lf\t", ue_data.GetSAD()); // ʂt@Cɏ

			if(m_ueData.SVM()){
				m_speeching = true;
				//*fprintf(wfp,"b茋 : 1(%lf)\n",ue_data.GetSVM()); // b茋ʂt@Cɏ
			}else{
				m_speeching = false;
				//*fprintf(wfp,"b茋 : 0(%lf)\n",ue_data.GetSVM()); // b茋ʂt@Cɏ
			}
			// f̕בւ
			m_ueData.sort_Data();
		}
		// ̕ۑ
		m_fdBefore = m_fd;

		if (m_beforeImage == NULL) {
			m_beforeImage = cvCreateImage(cvSize(captureImage->width, captureImage->height),
				captureImage->depth, captureImage->nChannels);
		}
		// 摜̃Rs[
		cvCopy(captureImage, m_beforeImage, NULL);
	}
	else {
		m_speeching = false;
	}

	// ͌ʂɃ\bhĂяo
	if (!CORBA:: is_nil(m_speechDetector._ptr())) {
		CORBA::Long currentFrame(m_currentFrame);
		CORBA::Long mouseX(m_fd.GetMouth().x), mouseY(m_fd.GetMouth().y), mouseWidth(m_fd.GetMouth().width), mouseHeight(m_fd.GetMouth().height); 
		CORBA::Float of(m_ueData.GetOF()), sad(m_ueData.GetSAD()), result(m_ueData.GetSVM());
		m_speechDetector->onAnalyze(currentFrame, mouseX, mouseY, mouseWidth, mouseHeight, of, sad, result);
		//cout << "[" << m_currentFrame << "]: " << process_time << "ms" << endl; // ԕ\
		// bJnĨ\bhĂяo
		if (prev_speeching != m_speeching) {
			if (m_speeching) {
				cout << "speechBegin" << endl;
				m_speechDetector->speechBegin();
			}
			else {
				cout << "speechEnd" << endl;
				m_speechDetector->speechEnd();
			}
		}
	}

	double process_time = (cvGetTickCount() - start_tick) / cvGetTickFrequency() / 1000.0;
	//cout << "[" << m_currentFrame << "]: " << process_time << "ms" << endl; // ԕ\

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t SpeechDetectorConsumer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t SpeechDetectorConsumer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t SpeechDetectorConsumer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t SpeechDetectorConsumer::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t SpeechDetectorConsumer::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
	void SpeechDetectorConsumerInit(RTC::Manager* manager)
	{
		coil::Properties profile(speechdetectorconsumer_spec);
		manager->registerFactory(profile,
			RTC::Create<SpeechDetectorConsumer>,
			RTC::Delete<SpeechDetectorConsumer>);
	}
};
